@@ -17,10 +17,9 @@
 import os
 import subprocess
 from distutils import ccompiler
-from distutils.util import get_platform
 
 from .c_code import LSODA_PATH
-from .utils import base_cache_folder, create_folder_if_not_exists
+from .utils import base_cache_folder, create_folder_if_not_exists, get_platform
 
 CC = ccompiler.new_compiler().executables["compiler"][0]
 HERE = os.path.dirname(os.path.abspath(__file__))
