@@ -16,6 +16,7 @@
 
 import json
 import os
+import pathlib
 from hashlib import md5
 
 import portalocker
@@ -41,20 +42,13 @@ def known_traces_file(unique_id):
 
 def _load_traces(path):
     with portalocker.Lock(path, "r", timeout=1, encoding="utf-8") as fh:
-        try:
-            traces = json.load(fh)
-        except json.decoder.JSONDecodeError:
-            # sometimes previos load fails in parallel executions!?
-            fh.seek(0)
-            content = repr(fh.read())
-            traces = json.loads(content)
-
-    return {int(k): v for (k, v) in traces.items()}
+        return {int(k): v for (k, v) in json.load(fh).items()}
 
 
 def _write_json(path, data):
-    with portalocker.Lock(path, "w", timeout=1, encoding="utf-8") as fh:
+    with portalocker.Lock(path, "w", encoding="utf-8", flags=portalocker.LOCK_EX) as fh:
         json.dump(data, fh, indent=4)
+        fh.flush()
 
 
 def read_known_traces(unique_id):
@@ -66,12 +60,28 @@ def read_known_traces(unique_id):
 
 def write_known_traces(unique_id, traces):
     path = known_traces_file(unique_id)
-    _write_json(path, traces)
+    with portalocker.Lock(path, "w", encoding="utf-8", flags=portalocker.LOCK_EX) as fh:
+        json.dump(traces, fh, indent=4)
+        fh.flush()
 
 
-def write_new_traces(unique_id, new_traces):
+def update_new_traces(unique_id, new_traces):
     path = new_traces_file(unique_id)
-    _write_json(path, new_traces)
+    with portalocker.Lock(
+        path + ".lock", "w", encoding="utf-8", flags=portalocker.LOCK_EX
+    ) as fh:
+        if os.path.exists(path):
+            with open(path, "r") as fh:
+                traces = json.load(fh)
+        else:
+            traces = {}
+
+        for k, v in new_traces.items():
+            traces.setdefault(k, []).extend(
+                vi for vi in v if vi not in traces.get(k, [])
+            )
+        with open(path, "w") as fh:
+            json.dump(traces, fh, indent=4)
 
 
 def new_traces_file(unique_id):
@@ -97,33 +107,6 @@ def read_new_traces(unique_id):
     return traces
 
 
-def update_new_traces(unique_id, new_traces):
-    if not new_traces:
-        return
-
-    num_levels = len(new_traces)
-
-    path = new_traces_file(unique_id)
-    if os.path.exists(path):
-        existing_traces = _load_traces(path)
-    else:
-        existing_traces = {}
-
-    if not existing_traces:
-        existing_traces = {level: list() for level in range(num_levels)}
-
-    did_update = False
-    for level, nti in new_traces.items():
-        for trace in nti:
-            if trace not in existing_traces[level]:
-                existing_traces[level].append(trace)
-                did_update = True
-
-    _write_json(path, existing_traces)
-    if did_update:
-        print("updated new traces at", path)
-
-
 class OdeFast:
     def __init__(self, name, t, lhs, rhs, splits=None, reorder=False):
         assert len(lhs) == len(rhs)
@@ -260,8 +243,8 @@ class OdeFastWrapper(WrapperBase):
 
     @concat_generator_results
     def cython_code(self, header_file_path):
-        here = os.path.dirname(__file__)
-        template = open(os.path.join(here, CYTHON_TEMPLATE)).read()
+        here = pathlib.Path(__file__).parent
+        template = (here / CYTHON_TEMPLATE).read_text()
 
         liw = 20 + self.n
 
